<?php
if(!defined('_CLASS_SESSION')){
   define('_CLASS_SESSION', '_CLASS_SESSION');

define('SESSION_CLIENT', 0);
define('SESSION_ADMIN', 1);
define('SESSION_CODE', 2);
define('SESSION_RETRY', 50);
define('SESSION_TIMER', 200000);

class Session {
	var $temp_folder;
	var $sessid;
	var $timeout = 1800;
	var $enable_cookies = true;
	var $enable_ipcheck = false;
	var $sesstype = SESSION_CLIENT;		//0 - client, 1 - admin, 2 - code
	var $delete_time = 43200;   		//12*3600
	
	function _Encode($strText, $key) {
//		return base64_encode($strText);

		$keylen = strlen($key);
		$iLen = strlen($strText);
		for ($i = 0; ($i < 1024) && ($i < $iLen); $i++) {			
			$ch = ord($strText[$i]);
			$strText[$i] = chr($ch ^ $key[$i%$keylen]);
		}
	
		return base64_encode($strText);
	}
	
	function _Decode($strText, $key) {
//		return base64_decode($strText);

		$strText = base64_decode($strText);
		
		$keylen = strlen($key);
		$iLen = strlen($strText);
		for ($i = 0; ($i < 1024) && ($i < $iLen); $i++) {
			$ch = ord($strText[$i]);
			$strText[$i] = chr($ch ^ $key[$i%$keylen]);
		}
		
		return $strText;
	}

	function GenSessionID() {
		$i = 0;
		$extname = ($this->sesstype == SESSION_CODE) ? '.code' : '.sess';
		
		while(1) {
			if ($this->sesstype == SESSION_ADMIN)
				$sessid = sha1(uniqid(microtime()));
			else
				$sessid = md5(uniqid(microtime()));
			
			$sessid = strtolower($sessid);
			
			$sessionfile = $this->temp_folder.'_sessions\\'.md5($sessid).$extname;

			if (!file_exists($sessionfile)) 
				return $sessid;

			if ($i++ >= SESSION_RETRY)
				return false;
				
			usleep(SESSION_TIMER);
		}
		
		return false;
	}
	
	function Load($chkfile = false) {
		if ($this->sessid == '')
			return false;
		
		$extname = ($this->sesstype == SESSION_CODE) ? '.code' : '.sess';
		$sessionfile = $this->temp_folder.'_sessions\\'.md5($this->sessid).$extname;

		if ($chkfile) {
			$i = 0;
			while (1) {
				if (file_exists($sessionfile))
					break;
					
				if ($i++ >= SESSION_RETRY)
					return false;
					
				usleep(SESSION_TIMER);
			}
		}

		if(!file_exists($sessionfile)) 
			return false;

		$content = file_get_contents($sessionfile);
		if (empty($content))
			return false;

		$data = $this->_Decode($content, $this->sessid);
		$result = unserialize($data);

		unset($content);
		unset($decode);
		
		if ($this->enable_cookies && $result['access-session-cookies']){
			$sessname = ($this->sesstype == SESSION_ADMIN) ? 'magicwinmail_admin_sessid' : 'magicwinmail_sessid';
			$cookiename = $sessname.'_'.substr(sha1($this->sessid), 0, 20);			
			$cookievalue = $_COOKIE[$cookiename];
			if (empty($cookievalue))
				return false;
			
			$pos = strpos($cookievalue, '.');
			if ($pos === false)
				return false;
			
			$timestamp = substr($cookievalue, 0, $pos);
			$chksum = substr($cookievalue, $pos+1);
	
			if (time()-$timestamp > $this->timeout || sha1(md5($timestamp).$result['access-cookies-chksum']) != $chksum)
				return false;
		}
		
		if ($this->enable_ipcheck) {
			if ($result['access-client-ipaddress'] != $_SERVER["REMOTE_ADDR"]) {
				$ret = false;
				$pos = strrpos($_SERVER["REMOTE_ADDR"], '.');
				if ($pos === false) 
					return false;
					
				if (strncmp($result['access-client-ipaddress'], $_SERVER["REMOTE_ADDR"], $pos+1) != 0)
					return false;
			}
		}

		return $result;
	}

	function Save(&$sess){
		$sessiondir = $this->temp_folder.'_sessions\\';
		if(!file_exists($sessiondir)) 
			mkdir($sessiondir, 0777);

		$extname = ($this->sesstype == SESSION_CODE) ? '.code' : '.sess';
		$sessionfile = $sessiondir.md5($this->sessid).$extname;
		
		$sess['access-client-ipaddress'] = $_SERVER["REMOTE_ADDR"];
		$sess['access-session-cookies'] = $this->enable_cookies;
		if (!isset($sess['access-cookies-chksum']))
			$sess['access-cookies-chksum'] = sha1(uniqid(microtime()));
		
		$data = serialize($sess);
		$content = $this->_Encode($data, $this->sessid);
		
		file_put_contents($sessionfile, $content);
		
		unset($data);
		unset($content);

		if ($this->enable_cookies) {
			$sessname = ($this->sesstype == SESSION_ADMIN) ? 'magicwinmail_admin_sessid' : 'magicwinmail_sessid';
			$cookiename = $sessname.'_'.substr(sha1($this->sessid), 0, 20);
			$cookievalue = time().'.'.sha1(md5(time()).$sess['access-cookies-chksum']);
			my_setcookie($cookiename, $cookievalue); 
		}

		return true;
	}

	function Kill() {
		$fileid = md5($this->sessid);
		$extname = ($this->sesstype == SESSION_CODE) ? '.code' : '.sess';
		$sessionfile = $this->temp_folder.'_sessions\\'.$fileid.$extname;
		@unlink($sessionfile);
		
		$sessiondir = $this->temp_folder.'_sessions\\'.$fileid.'_*';
		$arrfiles = glob($sessiondir);
		if (is_array($arrfiles)) {
			foreach ($arrfiles as $file) {
				if (is_file($file))
					@unlink($file);
			}
		}
	
		if ($this->enable_cookies) {
			$sessname = ($this->sesstype == SESSION_ADMIN) ? 'magicwinmail_admin_sessid' : 'magicwinmail_sessid';
			$cookiename = $sessname.'_'.substr(sha1($this->sessid), 0, 20);
			my_setcookie($cookiename, '');
		}
	}
	
	function Clean() {
		clearstatcache();
		
		$nowtime = time();
		
		$num = 0;
		$sessiondir = $this->temp_folder.'_sessions';
		$dh = opendir($sessiondir);
		if ($dh) {
			while ($file = readdir($dh)) {
				if ($file != '.' && $file != '..') {
					$extname = substr($file, -4);
					if ($extname == '.dat')
						continue;
										
					$thisfile = $sessiondir.'\\'.$file;
					if (!is_file($thisfile))
						continue;
						
					if ($num++ > 500)
						break;
					
					$mtime = @filemtime($thisfile);
					if (!$mtime)
						continue;
						
					$idle = intval($nowtime-$mtime);
					if (($extname == 'code' && $idle > 600)
						|| $idle > $this->delete_time) {
						@unlink($thisfile);
					}
				}
			}
					
			closedir($dh);
		}
		
		foreach ($_COOKIE as $key => $value) {
			if (strncmp($key, 'magicwinmail_sessid_', 20) == 0
				|| strncmp($key, 'magicwinmail_admin_sessid_', 26) == 0) {
				$idle = $nowtime-intval($value);
				if ($idle > $this->delete_time) {
					my_setcookie($key, '');
				}
			}
		}
	}
}

}
?>